static char *cvs_id =" $Id: xnd_pras.c,v 1.1 2004/07/30 08:04:23 berar Exp $ ";
/* XND_PRAS.C   asymmetry profile functions */

#define  USE_BRAGG
#include "xnd_prof.h"
/* =========================================================== */
/*   f_asym return                             */
/*         as_coef = 1 + eps                    */
/*   and calculates                             */
/*      as_deriv     derivative  with t         */
/*      as_d[i]      derivative for each coef   */
/* -------------------------------------------- */

/* asymetry function and values used in its derivatives */
/*  ancienne definition version 1.00 1.01  
/*     as_da = dt_w;
/*		 as_db = dt_w * abs_dt;
/*		 as_val =  act->asym + 2 * act->bssym * abs_dt;
/* 1.02 utilisation des polynomes d'Hermite impair  1 et 3 
/* 1.07 utilisation des polynomes d'Hermite impair 1, 3 ... MAX_ASYM */
/* F(0)=H(1) = -2x                                                    */
/* F(1)=H(3) = x*(12-8x^2)                                            */
/* F(2)=H(5)=  x*(-120+160x^2-32x^4)                                  */
/* F(3)=H(7)=  x*(1680-3360x^2+1344x^4-128x^6)                        */
/* F(4)=H(9)=  x*(-30240+80640x^2-48384x^4+9216x^6-512x^8)            */
/* F(5)=H(11)= x*(665280-2217600x^2+1774080x^4-506880x^6+56320x^8-2048x^10) */
/* F'(0)=H(2)= -2+4x^2                                                */
/* F'(1)=H(4)= 12-48x^2+16x^4                                         */
/* F'(2)=H(6)= -120+720x^2-480x^4+64x^6                               */
/* F'(3)=H(8)= 1680-13440x^2+13440x^4-358464x^6+256x^8                */
/* F'(4)=H(10)= -30240+302400x^2-403200x^4+161280x^6-23040x^8+1024x^10 */
/* F'(5)=H(12)= 665280-7983360x^2+13305600x^4-7096320x^6+1520640x^8-135168x^10+4096x^12 */
/*
/* version 1.10 plusieurs possibilites          */
/*  max_asym >0 : polynomes d'hermite  cf 1.07 */
/*           <0 : t^(2n+1)e^-t^2               */
/* =========================================================== */
float asym_herm (P_(void))
{   float sdt, dt_w, val;
	 float dherm[MAX_ASYM];
	 int i;
	 dt_w = dt / w_asym;
	 sdt = dt_w * dt_w;
	 val = exp(-sdt);
	 switch (max_asym )
	  {   case 6 : as_d[5]  = 665280.0-sdt*(2217600.0-sdt*(1774080.0-sdt*(506880.0-sdt*(56320.0-sdt*2048.0))));
						dherm[2] = 665280.0-sdt*(7983360.0-sdt*(13305600.0-sdt*(7096320.0-sdt*(1520640.0-sdt*(135168.0-sdt*4096.0)))));
			case 5 : as_d[4]  = -30240.0+sdt*(80640.0-sdt*(48384.0-sdt*(9216.0-sdt*512.0)));
						dherm[2] = -30240.0+sdt*(302400.0-sdt*(403200.0-sdt*(161280.0-sdt*(23040.0-sdt*1024.0))));
			case 4 : as_d[3]  = 1680.0-sdt*(3360.0-sdt*(1344.0-sdt*128.0));
						dherm[2] = 1680.0-sdt*(13440.0-sdt*(13440.0-sdt*(358464.0-sdt*256.0)));
			case 3 : as_d[2]  = -120.0+sdt*(160.0-sdt*32.0);
						dherm[2] = -120.0+sdt*(720.0-sdt*(480.0-sdt*64.0));
			case 2 : as_d[1]  = 12.0-sdt*8.0;
						dherm[1] = 12.0-sdt*(48.0-sdt*16.0 );
			case 1 : as_d[0]  = -2.0;
						dherm[0] = -2.0+4.0*sdt;
		 }
	 as_coef = 1;
	 as_deriv = 0;
	 for ( i=0; i < max_asym; i++ )
	 {   as_d[i] *= dt_w * val;
		  as_deriv += act->asym[i] * dherm[i];
        as_coef  += act->asym[i] * as_d[i];
        }
    as_deriv *= val / w_asym;    
	 return as_coef ;
	 }
/* ----------------------------------------------------------- */
float asym_power (P_(void))
{   float sdt, dt_w, val;
    float dherm[MAX_ASYM];
    int i;
	 dt_w = dt / w_asym;
	 sdt = dt_w * dt_w;
	 val = exp(-sdt);
    switch (max_asym )
	  {   case 6 : as_d[5]  = sdt*sdt*sdt*sdt*sdt;
                  dherm[2] = sdt*sdt*sdt*sdt*(9.0-2.0-sdt);
			case 5 : as_d[4]  = sdt*sdt*sdt*sdt;
						dherm[2] = sdt*sdt*sdt*(7.0-2.0*sdt);
			case 4 : as_d[3]  = sdt*sdt*sdt;
						dherm[2] = sdt*sdt*(5.0-2.0*sdt);
			case 3 : as_d[2]  = sdt*sdt;
                  dherm[2] = -120.0+sdt*(720.0-sdt*(480.0-sdt*64.0));
			case 2 : as_d[1]  = sdt;
						dherm[1] = sdt*(3.0-2.0*sdt);
			case 1 : as_d[0]  = 1.0;
						dherm[0] = 1.0-2.0*sdt;
       }
    as_coef = 1;
    as_deriv = 0;
    for ( i=0; i < max_asym; i++ ) 
	 {   as_d[i] *= dt_w * val;
        as_deriv += act->asym[i] * dherm[i];
        as_coef  += act->asym[i] * as_d[i];
        }
    as_deriv *= val / w_asym;    
	 return as_coef ;
	 }
/* ----------------------------------------------------------- */
/*  coef0.t.|t|^coef1 .e^-t^2                                  */
float asym_adj (P_(void))
{   float sdt, dt_w, val;
    float a, b, lsdt;
	 dt_w = dt / w_asym;
	 sdt = dt_w * dt_w;
	 if ( sdt > MINFLOAT )
    {  val = exp(-sdt);
       lsdt = log(sdt);
       a =  -sdt + 0.5*act->asym[1]*lsdt;
       b = exp(a);
       as_d[0] = dt_w * b;
       as_d[1] = as_d[0] * lsdt * 0.5 ;
       as_coef = 1 + act->asym[0] * as_d[0];
	    as_deriv = 1 - 2.0*sdt;
	    as_deriv += dt_w >= 0 ? 1 : -1;
	    as_deriv *= b * val / w_asym;
       } else
     {  as_deriv = 0;
        as_coef = 1;
        }
	 return as_coef ;
	 }
/* ----------------------------------------------------------- */
